#!/usr/bin/tclsh

source ~andi/bin/fileformat.tcl
set opencount 0
set dataext ".dat.bz2"

# Definition der Plotfarben
set color(q1) 1; set pointstyle(q1) 6
set color(q2) 1; set pointstyle(q2) 1
set color(qf) 1
set color(tra1) 4; set pointstyle(tra1) 6
set color(tra2) 4; set pointstyle(tra2) 1
set color(traf) 3
set color(uhold) 3
set color(frq1) 2; set pointstyle(frq1) 4
set color(frq2) 2; set pointstyle(frq2) 1
set color(frqf) 1
set color(level) 8; set pointstyle(level) 1
set color(pulse) 9; set pointstyle(pulse) 6
set color(pulse1) 2; set pointstyle(pulse1) 6
set color(pulse2) 4; set pointstyle(pulse2) 6
set color(fil1res) 2;
set color(fil2res) 4;
set color(subsres) 3; set pointstyle(subsres) 8
set color(pressure) 2;
set color(baratron) 1;
set color(bathtemp) 3;
set color(celltemp) 4;
set color(coilvolt) 2;
set color(coilamp)  1;

proc stamptotime {{timestamp ""}} {
	global dataext opencount
	if {[ catch { scan $timestamp "%d.%d.%d %d:%d:%d" day month year hours minutes seconds}]!=0} {
		puts "$timestamp"
	}
	if {[catch { set result [clock scan [format "%02d/%02d/%02d %02d:%02d:%02d" $month $day $year $hours $minutes $seconds]]}] != 0} {
		puts "ERROR: $timestamp"
		exit
	}
	return $result
}

proc parse_runlog {{gnuplot ""} {basename ""} {xbeg ""} {xend ""} {ymin 0} {ydelta 1} {axis 2}} {
	global dataext opencount
	set starttime [stamptotime $xbeg]
	set stoptime [stamptotime $xend]

	set runlogfile [open "$basename.runlog" "r"]
	incr opencount

	set labely [expr $ymin+0.5*$ydelta]
	set arrowy1 [expr $ymin+0.4*$ydelta]
	set arrowy2 [expr $ymin+0.1*$ydelta]

	set labelcount 0
	while {![eof $runlogfile]} {
		gets $runlogfile runlogline
		if {[string trim $runlogline]!=""} { 
			set messagetime "[lindex $runlogline 2] [lindex $runlogline 3]"
			set messagesecs [stamptotime $messagetime]

			if {$messagesecs > $starttime && $messagesecs < $stoptime} {
				incr labelcount
				if {$axis==2} {
					puts $gnuplot "set label $labelcount \"[lindex $runlogline 4]\" at \"$messagetime\", second $labely\
									rotate font 'Helvetica,7'" 
					puts $gnuplot "set arrow $labelcount from \"$messagetime\",second $arrowy1 to \"$messagetime\",second $arrowy2"
				} else {
					puts $gnuplot "set label $labelcount \"[lindex $runlogline 4]\" at \"$messagetime\",$labely\
									rotate font 'Helvetica,7'" 
					puts $gnuplot "set arrow $labelcount from \"$messagetime\",$arrowy1 to \"$messagetime\",$arrowy2"
				}
			}
		}
	}
	close $runlogfile
	incr opencount -1
}

proc get_columns {{basename ""}} {
	global dataext opencount
	set datafile [open "|bzcat $basename$dataext" "r"]
	incr opencount

	gets $datafile dataline
	set columns [llength $dataline]
	catch {close $datafile}
	incr opencount -1

	return $columns
}

proc plot_AB {{basename ""} {xbeg ""} {xend ""} {comment ""}} {
	global dataext
	plot_AB_core "1" $basename $xbeg $xend $comment
}

proc plot_AB2 {{basename ""} {xbeg ""} {xend ""} {comment ""}} {
	global dataext
	plot_AB_core "2" $basename $xbeg $xend $comment
}


proc plot_AB_core {{chan ""} {basename ""} {xbeg ""} {xend ""} {comment ""}} {
	global dataext opencount
	global col color pointstyle plotborder plotnumber plotindex;
	set columns [get_columns $basename]
	set columns [get_columns $basename]

	set datafilename "$basename$dataext"
	set toifilename "$basename.toi"
	set epsfile "$basename[]_[format "%03d%02d" $plotnumber $plotindex].eps"

	if {([file exists $epsfile] && [file mtime $toifilename] < [file mtime $epsfile]) && [file mtime $datafilename] < [file mtime $epsfile]} {
		puts "    skipping AB-data ($xbeg - $xend) $comment"
		return
	}
	
	puts "  scanning data ($xbeg - $xend) $comment"	
	scan_data $basename $xbeg $xend "uhold"
	
	scan [file tail $basename] "mw%s" filenumber
	set basefitname [file join [file dirname $basename] "mw1_$filenumber"]

	puts "    plotting AB-data of channel $chan ($xbeg - $xend) $comment"	
	set gnuplot [open "|gnuplot" w]
	incr opencount

	puts $gnuplot "set term post eps enhanced color solid"
	puts $gnuplot "set encoding iso"
	puts $gnuplot "set pointsize 0.2"
	puts $gnuplot "set ylabel \"transmission \[dB\]\""
	puts $gnuplot "set y2label \"frequency \[GHz\]\""
	puts $gnuplot "set output \"$epsfile\""
	puts $gnuplot "set multiplot"
	puts $gnuplot "set ytics nomirror"
	puts $gnuplot "set y2tics"
	puts $gnuplot "set rmargin 11"
	puts $gnuplot "set lmargin 10"
	puts $gnuplot "set size 1,0.5"
	puts $gnuplot "set origin 0,0.5"
	puts $gnuplot "set xr \[$col(uhold:min):$col(uhold:max)\]"
	puts $gnuplot "set xlabel \"holding voltage \[U\]\""
	if {[get_column $columns "frq1"]} {
		if {[get_column $columns "lin1"]!=0} {
			puts $gnuplot "plot \"<bzcat $datafilename\" every :::$xbeg[]::$xend u 10:(\$[get_column $columns "frq$chan"]/1e9) axes x1y2 ti \"frequency$chan\" w p $color(frq$chan) $pointstyle(frq$chan),\
								\"\" every :::$xbeg[]::$xend u 10:(log(\$[get_column $columns "amp$chan"]+\$[get_column $columns "off$chan"]+\$[get_column $columns "lin$chan"]/1e9*\$[get_column $columns "frq$chan"]/1e9)/log(10)*20) ti \"transmission$chan\" w p $color(tra$chan) $pointstyle(tra$chan)"
		} else {
			puts $gnuplot "plot \"<bzcat $datafilename\" every :::$xbeg[]::$xend u 10:(\$[get_column $columns "frq$chan"]/1e9) axes x1y2 ti \"frequency$chan\" w p $color(frq$chan) $pointstyle(frq$chan),\
								\"\" every :::$xbeg[]::$xend u 10:(log(\$[get_column $columns "amp$chan"])/log(10)*20) ti \"transmission\" w p $color(tra$chan) $pointstyle(tra$chan)"
		}
		puts $gnuplot "set origin 0,0"
		puts $gnuplot "set ytics mirror"
		puts $gnuplot "set noy2tics"
		puts $gnuplot "set y2label \"\""
		puts $gnuplot "set ylabel \"quality factor\""
		puts $gnuplot "plot \"<bzcat $datafilename\" every :::$xbeg[]::$xend u 10:(\$[get_column $columns "frq$chan"]/\$[get_column $columns "wid$chan"]) ti \"quality factor$chan\" w p $color(q$chan) $pointstyle(q$chan)"
	} elseif {[get_column $columns "frqmax$chan"]} {
			puts $gnuplot "set origin 0,0"
			puts $gnuplot "set size 1,1"
			puts $gnuplot "plot \"<bzcat $datafilename\" every :::$xbeg[]::$xend u [get_column $columns "uhold"]:[get_column $columns "dbmax$chan"] ti \"transmission\" w p $color(tra$chan) $pointstyle(tra$chan),\
							\"\" every :::$xbeg[]::$xend u [get_column $columns "uhold"]:(\$[get_column $columns "frqmax$chan"]/1e9) axes x1y2 ti \"frequency$chan\" w p $color(frq$chan) $pointstyle(frq$chan)"
	}
	puts $gnuplot "set nomultiplot"
	catch {close $gnuplot}
	incr opencount -1
}

proc plot_CD {{basename ""} {xbeg ""} {xend ""} {comment ""}} {
	global dataext opencount
	global color pointstyle plotborder plotnumber plotindex;
	set columns [get_columns $basename]

	set datafilename "$basename$dataext"
	set toifilename "$basename.toi"
	set epsfile "$basename[]_[format "%03d%02d" $plotnumber $plotindex].eps"

	if {([file exists $epsfile] && [file mtime $toifilename] < [file mtime $epsfile]) && [file mtime $datafilename] < [file mtime $epsfile]} {
		puts "    skipping CD-data ($xbeg - $xend) $comment"
		return
	}

	puts "    plotting CD-data ($xbeg - $xend) $comment"	
	set gnuplot [open "|gnuplot" w]
	incr opencount

	puts $gnuplot "set term post eps enhanced color solid"
	puts $gnuplot "set encoding iso"
	puts $gnuplot "set pointsize 0.2"
	puts $gnuplot "set ylabel \"bath thermometer \[k{/Symbol W}\]\""
	puts $gnuplot "set y2label \"cell thermometer \[k{/Symbol W}\]\""
	puts $gnuplot "set output \"$epsfile\""
	puts $gnuplot "set multiplot"
	puts $gnuplot "set ytics nomirror"
	puts $gnuplot "set y2tics"
	puts $gnuplot "set rmargin 11"
	puts $gnuplot "set lmargin 10"
	puts $gnuplot "set size 1,0.5"
	puts $gnuplot "set origin 0,0.5"
	puts $gnuplot "set xdata time"
	puts $gnuplot "set timefmt \"%d.%m.%y %H:%M:%S\""

	if {[string length $xbeg]>5} {
		puts $gnuplot "set xr \[\"$xbeg\":\"$xend\"\]"
		puts $gnuplot "plot \"<bzcat $datafilename\" u 1:(\$[get_column $columns "bathtemp"]/1e3) ti \"bath thermometer\" w l $color(bathtemp),\
							\"\" u 1:(\$[get_column $columns "celltemp"]/1e3) axes x1y2 ti \"cell thermometer\" w l $color(celltemp)"
	} else {
		puts $gnuplot "plot \"<bzcat $datafilename\" every :::$xbeg[]::$xend u 1:(\$[get_column $columns "bathtemp"]/1e3) ti \"bath thermometer\" w l $color(bathtemp),\
							\"\" every :::$xbeg[]::$xend u 1:(\$[get_column $columns "celltemp"]/1e3) axes x1y2 ti \"cell thermometer\" w l $color(celltemp)"
	}
	puts $gnuplot "set origin 0,0"
	puts $gnuplot "set ylabel \"substrate resistance \[M{/Symbol W}\]\""
	puts $gnuplot "set y2label \"filament resistance \[{/Symbol W}\]\""
	if {[string length $xbeg]>5} {
		puts $gnuplot "plot \"<bzcat $datafilename\" u 1:(\$[get_column $columns "subsres"]/1e6) ti \"substrate resistance\" w l $color(subsres), \
						\"\" u 1:(\$[get_column $columns "fil1res"]) axes x1y2 ti \"filament 1\" w l $color(fil1res), \
						\"\" u 1:(\$[get_column $columns "fil2res"]) axes x1y2 ti \"filament 2\" w l $color(fil2res)"
	} else {
		puts $gnuplot "plot \"<bzcat $datafilename\" every :::$xbeg[]::$xend u 1:(\$[get_column $columns "subsres"]/1e6) ti \"substrate resistance\" w p $color(subsres), \
						\"\" every :::$xbeg[]::$xend u 1:(\$[get_column $columns "fil1res"]) axes x1y2 ti \"filament 1\" w p $color(fil1res), \
						\"\" every :::$xbeg[]::$xend u 1:(\$[get_column $columns "fil2res"]) axes x1y2 ti \"filament 2\" w p $color(fil2res)"
	}
	puts $gnuplot "set nomultiplot"
	catch {close $gnuplot}
	incr opencount -1
}

proc plot_CR {{basename ""} {xbeg ""} {xend ""} {comment ""}} {
	global dataext
	plot_CR_core "1" $basename $xbeg $xend $comment
}

proc plot_CR2 {{basename ""} {xbeg ""} {xend ""} {comment ""}} {
	global dataext
	plot_CR_core "2" $basename $xbeg $xend $comment
}

proc plot_CR_core {{chan ""} {basename ""} {xbeg ""} {xend ""} {comment ""}} {
	global dataext opencount
	global color pointstyle plotborder plotnumber plotindex;
	set columns [get_columns $basename]

	set datafilename "$basename$dataext"
	set toifilename "$basename.toi"
	set epsfile "$basename[]_[format "%03d%02d" $plotnumber $plotindex].eps"

	if {([file exists $epsfile] && [file mtime $toifilename] < [file mtime $epsfile]) && [file mtime $datafilename] < [file mtime $epsfile]} {
		puts "    skipping CR-data ($xbeg - $xend) $comment"
		return
	}

	puts "    plotting CR-data ($xbeg - $xend) $comment"	
	set gnuplot [open "|gnuplot" w]
	incr opencount

	puts $gnuplot "set term post eps enhanced color solid"
	puts $gnuplot "set encoding iso"
	puts $gnuplot "set pointsize 0.2"
	puts $gnuplot "set ylabel \"transmission \[dB\]\""
	puts $gnuplot "set y2label \"frequency \[GHz\]\""
	puts $gnuplot "set output \"$epsfile\""
	puts $gnuplot "set multiplot"
	puts $gnuplot "set ytics nomirror"
	puts $gnuplot "set y2tics"
	puts $gnuplot "set rmargin 11"
	puts $gnuplot "set lmargin 10"
	puts $gnuplot "set size 1,0.5"
	puts $gnuplot "set origin 0,0.5"
	puts $gnuplot "set xr \[0:0.9\]"
	puts $gnuplot "set xlabel \"magnetic field \[T\]\""
	if {[get_column $columns "lin1"]!=0} {
		puts $gnuplot "plot \"<bzcat $datafilename\" every :::$xbeg[]::$xend u (\$[get_column $columns "coil_current"]*0.0401):(\$[get_column $columns "frq$chan"]/1e9) axes x1y2 ti \"frequency$chan\" w p $color(frq$chan) $pointstyle(frq$chan),\
							\"\" every :::$xbeg[]::$xend u (\$[get_column $columns "coil_current"]*0.0401):(log(\$[get_column $columns "amp$chan"]+\$[get_column $columns "off$chan"]+\$[get_column $columns "lin$chan"]/1e9*\$[get_column $columns "frq$chan"]/1e9)/log(10)*20) ti \"transmission$chan\" w p $color(tra$chan) $pointstyle(tra$chan)"
	} else {
		puts $gnuplot "plot \"<bzcat $datafilename\" every :::$xbeg[]::$xend u (\$[get_column $columns "coil_current"]*0.0401):(\$[get_column $columns "frq$chan"]/1e9) axes x1y2 ti \"frequency$chan\" w p $color(frq$chan) $pointstyle(frq$chan),\
							\ \"\" every :::$xbeg[]::$xend u (\$[get_column $columns "coil_current"]*0.0401):(log(\$[get_column $columns "amp$chan"])/log(10)*20) ti \"transmission$chan\" w p $color(tra$chan) $pointstyle(tra$chan)"
	}
	puts $gnuplot "set origin 0,0"
	puts $gnuplot "set ytics mirror"
	puts $gnuplot "set noy2tics"
	puts $gnuplot "set y2label \"\""
	puts $gnuplot "set ylabel \"quality factor\""
	puts $gnuplot "plot \"<bzcat $datafilename\" every :::$xbeg[]::$xend u (\$[get_column $columns "coil_current"]*0.0401):(\$[get_column $columns "frq$chan"]/\$[get_column $columns "wid$chan"]) ti \"quality factor$chan\" w p $color(q$chan) $pointstyle(q$chan)"
	puts $gnuplot "set nomultiplot"
	catch {close $gnuplot}
	incr opencount -1
}

proc plot_LC {{basename ""} {xbeg ""} {xend ""} {comment ""}} {
	global dataext opencount
	global color pointstyle plotborder plotnumber plotindex;
	set columns [get_columns $basename]

	set datafilename "$basename$dataext"
	set toifilename "$basename.toi"
	set epsfile "$basename[]_[format "%03d%02d" $plotnumber $plotindex].eps"

	if {([file exists $epsfile] && [file mtime $toifilename] < [file mtime $epsfile]) && [file mtime $datafilename] < [file mtime $epsfile]} {
		puts "    skipping LC-data ($xbeg - $xend) $comment"
		return
	}

	puts "    plotting LC-data ($xbeg - $xend) $comment"	
	set gnuplot [open "|gnuplot" w]
	incr opencount

	puts $gnuplot "set term post eps enhanced color solid"
	puts $gnuplot "set encoding iso"
	puts $gnuplot "set pointsize 0.2"
	puts $gnuplot "set ylabel \"transmission \[dB\]\""
	puts $gnuplot "set y2label \"frequency \[GHz\]\""
	puts $gnuplot "set output \"$epsfile\""
	puts $gnuplot "set multiplot"
	puts $gnuplot "set ytics nomirror"
	puts $gnuplot "set y2tics"
	puts $gnuplot "set rmargin 11"
	puts $gnuplot "set lmargin 10"
	puts $gnuplot "set size 1,0.5"
	puts $gnuplot "set origin 0,0.5"
	puts $gnuplot "set xlabel \"level capacitor \[pF\]\""
	puts $gnuplot "plot \"<bzcat $datafilename\" every :::$xbeg[]::$xend u (\$[get_column $columns "levelcap"]*1e12):(log(\$[get_column $columns "amp1"])/log(10)*20) ti \"transmission\" w p $color(tra1) $pointstyle(tra1),\
						\"\" every :::$xbeg[]::$xend u (\$[get_column $columns "levelcap"]*1e12):(\$[get_column $columns "frq1"]/1e9) axes x1y2 ti \"frequency\" w p $color(frq1) $pointstyle(frq1)"
	puts $gnuplot "set origin 0,0"
	puts $gnuplot "set ytics mirror"
	puts $gnuplot "set noy2tics"
	puts $gnuplot "set y2label \"\""
	puts $gnuplot "set ylabel \"quality factor\""
	puts $gnuplot "plot \"<bzcat $datafilename\" every :::$xbeg[]::$xend u (\$[get_column $columns "levelcap"]*1e12):(\$[get_column $columns "frq1"]/\$[get_column $columns "wid1"]) ti \"quality factor\" w p $color(q1) $pointstyle(q1)"
	puts $gnuplot "set nomultiplot"
	catch {close $gnuplot}
	incr opencount -1
}

proc plot_PO {{basename ""} {xbeg ""} {xend ""} {comment ""}} {
	global dataext opencount
	global color pointstyle plotborder plotnumber plotindex;
	set columns [get_columns $basename]

	set datafilename "$basename$dataext"
	set toifilename "$basename.toi"
	set epsfile "$basename[]_[format "%03d%02d" $plotnumber $plotindex].eps"

	if {([file exists $epsfile] && [file mtime $toifilename] < [file mtime $epsfile]) && [file mtime $datafilename] < [file mtime $epsfile]} {
		puts "    skipping PO-data ($xbeg - $xend) $comment"
		return
	}

	puts "    plotting PO-data ($xbeg - $xend) $comment"	
	set gnuplot [open "|gnuplot" w]
	incr opencount
	
	puts $gnuplot "set term post eps enhanced color solid"
	puts $gnuplot "set encoding iso"
	puts $gnuplot "set pointsize 0.2"
	puts $gnuplot "set ylabel \"transmission \[dB\]\""
	puts $gnuplot "set y2label \"frequency \[GHz\]\""
	puts $gnuplot "set output \"$epsfile\""
	puts $gnuplot "set multiplot"
	puts $gnuplot "set ytics nomirror"
	puts $gnuplot "set y2tics"
	puts $gnuplot "set rmargin 11"
	puts $gnuplot "set lmargin 10"
	puts $gnuplot "set size 1,0.5"
	puts $gnuplot "set origin 0,0.5"
#	puts $gnuplot "set xr \[0:0.9\]"
	puts $gnuplot "set xlabel \"microwave power \[dBm\]\""
	if {[get_column $columns "lin1"]!=0} {
		if {[get_column $columns "napower"]==0} {
			set col "1"
		} else {
			set col ""
		}
		puts -nonewline $gnuplot "plot \"<bzcat $datafilename\" every :::$xbeg[]::$xend u [get_column $columns "napower$col"]:(\$[get_column $columns "frq1"]/1e9) axes x1y2 ti \"frequency$col\" w p $color(frq1) $pointstyle(frq1),\
							\"\" every :::$xbeg[]::$xend u [get_column $columns "napower$col"]:(log(\$[get_column $columns "amp1"]+\$[get_column $columns "off1"]+\$[get_column $columns "lin1"]/1e9*\$[get_column $columns "frq1"]/1e9)/log(10)*20) ti \"transmission$col\" w p $color(tra1) $pointstyle(tra1)"
		if {$col!=""} {
			puts $gnuplot ", \"\" every :::$xbeg[]::$xend u [get_column $columns "napower2"]:(\$[get_column $columns "frq2"]/1e9) axes x1y2 ti \"frequency2\" w p $color(frq2) $pointstyle(frq2),\
							\"\" every :::$xbeg[]::$xend u [get_column $columns "napower2"]:(log(\$[get_column $columns "amp2"]+\$[get_column $columns "off2"]+\$[get_column $columns "lin2"]/1e9*\$[get_column $columns "frq2"]/1e9)/log(10)*20) ti \"transmission2\" w p $color(tra2) $pointstyle(tra2)"
		} else {
			puts $gnuplot ""
		}
	} else {
		puts $gnuplot "plot \"<bzcat $datafilename\" every :::$xbeg[]::$xend u [get_column $columns "napower"]:(\$[get_column $columns "frq1"]/1e9) axes x1y2 ti \"frequency\" w p $color(frq1) $pointstyle(frq1),\
							\ \"\" every :::$xbeg[]::$xend u [get_column $columns "napower"]:(log(\$[get_column $columns "amp1"])/log(10)*20) ti \"transmission\" w p $color(tra1) $pointstyle(tra1)"
	}
	puts $gnuplot "set origin 0,0"
	puts $gnuplot "set ytics mirror"
	puts $gnuplot "set noy2tics"
	puts $gnuplot "set y2label \"\""
	puts $gnuplot "set ylabel \"quality factor\""
	
	puts -nonewline $gnuplot "plot \"<bzcat $datafilename\" every :::$xbeg[]::$xend u [get_column $columns "napower$col"]:(\$[get_column $columns "frq1"]/\$[get_column $columns "wid1"]) ti \"quality factor$col\" w p $color(q1) $pointstyle(q1)"
	if {$col!=""} {
		puts $gnuplot ", \"\" every :::$xbeg[]::$xend u [get_column $columns "napower2"]:(\$[get_column $columns "frq2"]/\$[get_column $columns "wid2"]) ti \"quality factor2\" w p $color(q2) $pointstyle(q2)"
	} else {
		puts $gnuplot ""
	}
	puts $gnuplot "set nomultiplot"
	catch {close $gnuplot}
	incr opencount -1
}


proc plot_transres {{basename ""}} {
	global dataext opencount
	global color pointstyle plotborder plotnumber;
# Diese Subroutine ist fr das Plotten der Substratinformation zustndig

	set columns [get_columns $basename]
	
	set datafilename "$basename$dataext"
	set toifilename "$basename.toi"
	set epsfile "$basename[]_d.eps"

	if {([file exists $epsfile] && [file mtime $toifilename] < [file mtime $epsfile]) && [file mtime $datafilename] < [file mtime $epsfile]} {
		puts "  skipping substrate information ($columns columns)"
		return
	}

	puts "  plotting substrate information ($columns columns)"	
	set gnuplot [open "|gnuplot" w]
	incr opencount

	puts $gnuplot "set term post eps enhanced color solid"
	puts $gnuplot "set encoding iso"
	puts $gnuplot "set xlabel \"cell thermometer \[{/Symbol W}\]\""
	puts $gnuplot "set ylabel \"transmission \[dB\]\""
	if {[get_column $columns "subsres"]!=0} {
		puts $gnuplot "set y2label \"substrate resistance \[k{/Symbol W}\]\""
		puts $gnuplot "set ytics nomirror; set y2tics; set log y2"
	}
	puts $gnuplot "set xr \[0:100\]"
	puts $gnuplot "set output \"$epsfile\""
	if {[get_column $columns "lin1"]!=0} {
		puts "lin1 exists"
		puts -nonewline $gnuplot "plot \"<bzcat $basename$dataext\" u [get_column $columns "celltemp"]:(log(\$[get_column $columns "amp1"]+\$[get_column $columns "off1"]+\$[get_column $columns "lin1"]/1e9*\$[get_column $columns "frq1"]/1e9)/log(10)*20) ti \"transmission1\" w linesp $color(tra1) $pointstyle(tra1)"
		if {[get_column $columns "lin2"]!=0} {
			puts -nonewline $gnuplot ", \"\" u [get_column $columns "celltemp"]:(log(\$[get_column $columns "amp2"]+\$[get_column $columns "off2"]+\$[get_column $columns "lin2"]/1e9*\$[get_column $columns "frq2"]/1e9)/log(10)*20) ti \"transmission2\" w linesp $color(tra2) $pointstyle(tra2)"
		}
	} else {
		if {[get_column $columns "amp1"]!=0} {
			puts -nonewline $gnuplot "plot \"<bzcat $basename$dataext\" u [get_column $columns "celltemp"]:(log(\$[get_column $columns "amp1"])/log(10)*20) ti \"transmission1\" w linesp $color(tra1) $pointstyle(tra1)"
			if {[get_column $columns "amp2"]!=0} {
				puts -nonewline $gnuplot ", \"\" u [get_column $columns "celltemp"]:(log(\$[get_column $columns "amp2"])/log(10)*20) ti \"transmission2\" w linesp $color(tra2) $pointstyle(tra2)"
			}
		} else {
			puts -nonewline $gnuplot "plot \"<bzcat $basename$dataext\" u [get_column $columns "celltemp"]:[get_column $columns "dbmax1"] ti \"transmission1\" w linesp $color(tra1) $pointstyle(tra1)"
			if {[get_column $columns "dbmax2"]!=0} {
				puts -nonewline $gnuplot ", \"\" u [get_column $columns "celltemp"]:[get_column $columns "dbmax2"] ti \"transmission2\" w linesp $color(tra2) $pointstyle(tra2)"
			}
		}
	}
	if {[get_column $columns "subsres"]!=0} {
		puts $gnuplot ", \"\" u [get_column $columns "celltemp"]:(\$[get_column $columns "subsres"]/1000) axes x1y2 ti \"substrate resistance\" w linesp $color(subsres) $pointstyle(subsres)"
	} else {
		puts $gnuplot ""
	}
	
	incr opencount -1
	if {[catch {close $gnuplot}]!=0} {
		
		file delete $epsfile
		puts "    failed, try w/o subsres data"
		set gnuplot [open "|gnuplot" w]
		incr opencount

		puts $gnuplot "set term post eps enhanced color solid"
		puts $gnuplot "set encoding iso"
		puts $gnuplot "set xlabel \"cell thermometer \[{/Symbol W}\]\""
		puts $gnuplot "set ylabel \"transmission \[dB\]\""
		puts $gnuplot "set xr \[0:100\]"
		puts $gnuplot "set output \"$epsfile\""
		if {[get_column $columns "lin1"]!=0} {
			puts -nonewline $gnuplot "plot \"<bzcat $basename$dataext\" u [get_column $columns "celltemp"]:(log(\$[get_column $columns "amp1"]+\$[get_column $columns "off1"]+\$[get_column $columns "lin1"]/1e9*\$[get_column $columns "frq1"]/1e9)/log(10)*20) ti \"transmission1\" w linesp $color(tra1) $pointstyle(tra1)"
			if {[get_column $columns "lin2"]!=0} {
				puts -nonewline $gnuplot ", \"\" u [get_column $columns "celltemp"]:(log(\$[get_column $columns "amp2"]+\$[get_column $columns "off2"]+\$[get_column $columns "lin2"]/1e9*\$[get_column $columns "frq2"]/1e9)/log(10)*20) ti \"transmission2\" w linesp $color(tra2) $pointstyle(tra2)"
			}
		} else {
			if {[get_column $columns "amp1"]!=0} {
				puts -nonewline $gnuplot "plot \"<bzcat $basename$dataext\" u [get_column $columns "celltemp"]:(log(\$[get_column $columns "amp1"])/log(10)*20) ti \"transmission1\" w linesp $color(tra1) $pointstyle(tra1)"
				if {[get_column $columns "amp2"]!=0} {
					puts -nonewline $gnuplot ", \"\" u [get_column $columns "celltemp"]:(log(\$[get_column $columns "amp2"])/log(10)*20) ti \"transmission2\" w linesp $color(tra2) $pointstyle(tra2)"
				}
			} else {
				puts -nonewline $gnuplot "plot \"<bzcat $basename$dataext\" u [get_column $columns "celltemp"]:[get_column $columns "dbmax1"] ti \"transmission1\" w linesp $color(tra1) $pointstyle(tra1)"
				if {[get_column $columns "dbmax2"]!=0} {
					puts -nonewline $gnuplot ", \"\" u [get_column $columns "celltemp"]:[get_column $columns "dbmax2"] ti \"transmission2\" w linesp $color(tra2) $pointstyle(tra2)"
				}
			}
		}
		puts $gnuplot ""
		incr opencount -1
		if {[catch {close $gnuplot}]!=0} {
			file delete $epsfile
			puts "    failed again"
		}
	}
}

proc plot_level {{basename ""} {xbeg ""} {xend ""}} {
	global dataext opencount
	global color pointstyle plotborder plotnumber;
# Diese Subroutine ist fr das Plotten des Heliumlevels zustndig
	set columns [get_columns $basename]
	
	if {[get_column $columns "levelcap"]==0} {
		puts "  no level information for `[file tail $basename]` ($columns columns)"
		return
	}

	set datafilename "$basename$dataext"
	set toifilename "$basename.toi"
	set epsfile "$basename[]_level.eps"

	if {([file exists $epsfile] && [file mtime $toifilename] < [file mtime $epsfile]) && [file mtime $datafilename] < [file mtime $epsfile]} {
		puts "  skipping level information ($xbeg - $xend) ($columns columns)"
		return
	}

	puts "  plotting level information ($xbeg - $xend) ($columns columns)"	
	set gnuplot [open "|gnuplot" w]
	incr opencount

	puts $gnuplot "set term post eps enhanced color solid"
	puts $gnuplot "set encoding iso"
	puts $gnuplot "set xdata time"
	puts $gnuplot "set timefmt \"%d.%m.%y %H:%M:%S\""
	puts $gnuplot "set ylabel \"level capacitor \[pF\]\""
	puts $gnuplot "set xr \[\"$xbeg\":\"$xend\"\]"
	puts $gnuplot "set output \"$basename[]_level.eps"
	puts $gnuplot "plot \"<bzcat $basename$dataext\" u 1:(\$[get_column $columns "levelcap"]*1e12) w l $color(level)"
	catch {close $gnuplot}
	incr opencount -1
}

proc plot_levelcalib {{basename ""} {toiline ""}} {
	global dataext opencount
	global color pointstyle plotborder plotnumber;
# Diese Subroutine ist fr das Plotten des Heliumlevels zustndig
	set columns [get_columns $basename]

	foreach datacol "date time levelcap frq1" {
		set col($datacol) [get_column $columns "$datacol"]
	}
	
	if {$col(levelcap)==0} {
		puts "  no level information for `[file tail $basename]` ($columns columns)"
		if {$col(frq1)==0} {
			puts "  and no frequency information"
			return
		}
	}

	if {[llength $toiline] < 5} {
		return
	}
	
	set datafilename "$basename$dataext"
	set toifilename "$basename.toi"
	set epsfile "$basename[]_levelcalib.eps"

	if {([file exists $epsfile] && [file mtime $toifilename] < [file mtime $epsfile]) && [file mtime $datafilename] < [file mtime $epsfile]} {
		puts "  skipping levelcalib information"
		return
	}

	set plotindex 0

	set gnuplot [open "|gnuplot" w]
	incr opencount

	puts $gnuplot "set term post eps enhanced color solid"
	puts $gnuplot "set encoding iso"
	puts $gnuplot "set pointsize 0.3"
	puts $gnuplot "set output \"$basename[]_levelcalib.eps"
	puts $gnuplot "set xlabel \"level capacitor \[pF\]\""
	puts $gnuplot "set ylabel \"resonance frequency \[GHz\]\""


# zuerst mal die einzelnen Teile und deren Startzeit ausfindig machen:
	set numberofparts 0
	set toifile [open $toifilename "r"]
	incr opencount

	gets $toifile toiline2
	while {![eof $toifile]} {
		gets $toifile toiline2
		set toiline2 [string trim $toiline2]
		if {$toiline2!="" && [string first # $toiline2]!=0} {
			incr numberofparts
			set part($numberofparts:starttime) [stamptotime [lindex $toiline2 0]] 
			set part($numberofparts:done) 0
		}
	}
	close $toifile
	incr opencount -1

	set levelcapmax ""
	set levelcapmin ""
	set frq1max ""
	set frq1min ""

	if {$col(levelcap)==0} {
		set levelfuncs ""
	}
	
	set datafile [open "|bzcat $datafilename" "r"]
	incr opencount

	while {![eof $datafile]} {
		gets $datafile dataline
		
		if {[string trim $dataline] != "" && [string index [string trim $dataline] 0]!="#"} {
			set meastime [stamptotime "[lindex $dataline [expr $col(date)-1]]\
									   [lindex $dataline [expr $col(time)-1]]"]
									   
			for {set i 1} {$i<=$numberofparts} {incr i} {
				if {$part($i:starttime)<=$meastime && !$part($i:done)} {
					set startfreq [lindex $dataline [expr $col(frq1)-1]]
					if {$col(levelcap)!=0} {
						set startlevelcap [lindex $dataline [expr $col(levelcap)-1]]
					
						if {$startfreq != "?" && $startlevelcap != "?"} {
							scan $startfreq "%g" part($i:startfreq) 
							set part($i:startlevelcap) $startlevelcap
							set part($i:done) 1
							set xval [expr $part($i:startlevelcap)*1e12]
							set yval [expr $part($i:startfreq)/1e9]
							puts $gnuplot "set label $i \"#$i\" at $xval,$yval rotate font 'Helvetica,12'"
						}
					} else {
						if {$startfreq != "?"} {
							scan $startfreq "%g" part($i:startfreq) 
							set part($i:done) 1
							set xval 0
							set yval [expr $part($i:startfreq)/1e9]
							set levelfuncs "$levelfuncs, $yval not w l $color(level)"
							puts $gnuplot "set label $i \"#$i\" at $xval,$yval font 'Helvetica,12'"
						}
					}
				}
			}
		}
	}	

	while {[llength $toiline] >= 4+$plotindex*2} {
		if {$plotindex==0} {
			puts -nonewline $gnuplot "plot "
		} else {
			puts $gnuplot ", \\"
		}

		incr plotindex
		set xbeg [lindex $toiline [expr 1+2*$plotindex]]
		set xend [lindex $toiline [expr 2+2*$plotindex]]
		puts "    plotting levelcalib information ($xbeg - $xend)"
		if {$col(levelcap)!=0} {
			puts -nonewline $gnuplot "\"<bzcat $basename$dataext\" every :::$xbeg[]::$xend u (\$$col(levelcap)*1e12):(\$$col(frq1)/1e9) not w p $color(level) $pointstyle(level)"
		} else {
			puts -nonewline $gnuplot "\"<bzcat $basename$dataext\" every :::$xbeg[]::$xend u 0:(\$$col(frq1)/1e9) not w p $color(level) $pointstyle(level)"
		}
	}

	if {$col(levelcap)==0 && $levelfuncs!=""} {
		puts $gnuplot "$levelfuncs"
	}
	catch {close $gnuplot}
	incr opencount -1
}

proc scan_data {{basename ""} {xbeg ""} {xend ""} {testcols ""}} {
	global dataext opencount
	global uholdskip col;
	# 5% Platz beim UHalt-Plot
	set plotborder 0.05
	
	if {[string length $xbeg]>5} {
		set starttime [stamptotime $xbeg]
		set stoptime [stamptotime $xend]
	} else {
		set starttime $xbeg
		set stoptime $xend
	}
	
	set datafile [open "|bzcat $basename$dataext" "r"]
	incr opencount

	set linenumber 0

	set col(uhold:maxval) 400
	set col(uhold:minval) -90
	
	foreach datacol $testcols {
			set col($datacol:max) ""
			set col($datacol:maxl) 0
			set col($datacol:min) ""
			set col($datacol:minl) 0
			set col($datacol:valid) 0
	}
	
	set seccount 0
	while {![eof $datafile]} {
		gets $datafile dataline
		incr linenumber

		if {[string trim $dataline] == "" && ![eof $datafile]} {
			incr seccount
		}
		
		if {[string trim $dataline] != "" && [string index [string trim $dataline] 0]!="#"} {

			if {[string length $xbeg]>5} {	
				set meastime [stamptotime "[lindex $dataline [expr $col(date)-1]]\
							  [lindex $dataline [expr $col(time)-1]]"]
			} else {
				set meastime $seccount
			}
			
			if {$meastime>=$starttime && $meastime<=$stoptime} {
				foreach datacol $testcols {
#
# Diese Schleife findet fr die Spaltenvariablen, die in "testcols" definiert sind
# die Maximal- und Minimalwerte im Bereich "starttime"-"stoptime"
#
					
					if {$col($datacol) != 0} {
						set data [lindex $dataline [expr $col($datacol)-1]]
						
						if {$col($datacol:max)==""} {
							if {$data != "?" && $data != ""} {
								set col($datacol:valid) 1
								if {[info exists col($datacol:maxval)]} {
									if {[info exists col($datacol:minval)]} {
										if {$data < $col($datacol:maxval) && $data > $col($datacol:minval)} {
											set col($datacol:max) $data
											set col($datacol:min) $data
											set col($datacol:maxl) $linenumber
											set col($datacol:minl) $linenumber
										}
									} else {
										if {$data < $col($datacol:maxval)} {
											set col($datacol:max) $data
											set col($datacol:min) $data
											set col($datacol:maxl) $linenumber
											set col($datacol:minl) $linenumber
										}
									}
								} else {
									if {[info exists col($datacol:minval)]} {
										if {$data > $col($datacol:minval)} {
											set col($datacol:max) $data
											set col($datacol:min) $data
											set col($datacol:maxl) $linenumber
											set col($datacol:minl) $linenumber
										}
									} else {
											set col($datacol:max) $data
											set col($datacol:min) $data
											set col($datacol:maxl) $linenumber
											set col($datacol:minl) $linenumber
									}
								}
							}
						} elseif {$data != "?"} {
							if {$data > $col($datacol:max)} {
								if {[info exists col($datacol:maxval)]} {
									if {$data < $col($datacol:maxval)} {
										set col($datacol:max) $data
										set col($datacol:maxl) $linenumber
									}
								} else {
									set col($datacol:max) $data
									set col($datacol:maxl) $linenumber
								}
							} elseif {$data < $col($datacol:min)} {
								if {[info exists col($datacol:minval)]} {
									if {$data > $col($datacol:minval)} {
										set col($datacol:min) $data
										set col($datacol:minl) $linenumber
									}
								} else {
									set col($datacol:min) $data
									set col($datacol:minl) $linenumber
								}
							}
						}
					}
				}
			}
		}
	}
	
	if {$col(uhold:max) != ""} {
		set uholdskip [expr ($col(uhold:max)-$col(uhold:min))*$plotborder/(1.0-$plotborder)]
		if {$uholdskip==0.0} {
			set uholdskip 0.2
		}
		
		set col(uhold:max) [expr $col(uhold:max)+$uholdskip]
		set col(uhold:min) [expr $col(uhold:min)-$uholdskip]
	} else {
		set uholdskip [expr (-90-(-100))*$plotborder/(1.0-$plotborder)]
		set col(uhold:max) [expr -90+$uholdskip]
		set col(uhold:min) [expr -100-$uholdskip]
		puts "no uhold value available"
	}
	close $datafile
	incr opencount -1
}

proc plot_data {{basename ""} {xbeg ""} {xend ""} {comment ""}} {
	global dataext opencount
	global col uholdskip color pointstyle plotborder plotnumber;

	set columns [get_columns $basename]

	foreach datacol "date time bathtemp celltemp pressure baratron fil1res fil2res subsres coil_current coil_voltage uhold amp1 frq1 wid1 lin1 off1 amp2 frq2 wid2 lin2 off2 pulsenumber dbmax1 frqmax1 filused" {
		set col($datacol) [get_column $columns "$datacol"]
	}
	
	set datafilename "$basename$dataext"
	set toifilename "$basename.toi"
	set runlogfilename "$basename.runlog"
	
	if {$plotnumber==0} {
		set eps1file "$basename[]_a.eps"
		set eps2file "$basename[]_b.eps"
		set eps3file "$basename[]_c.eps"
	} else {
		set eps1file "$basename[]_[format "%03d" $plotnumber]_a.eps"
		set eps2file "$basename[]_[format "%03d" $plotnumber]_b.eps"
		set eps3file "$basename[]_[format "%03d" $plotnumber]_c.eps"
	}
	
	if {([file exists $eps1file]  && [file mtime $toifilename] < [file mtime $eps1file]) && [file mtime $datafilename] < [file mtime $eps1file]} {
		if {!([file exists $runlogfilename] && [file mtime $runlogfilename] > [file mtime $eps1file])} {
			set do_dataplot1 0
			set do_dataplot2 0
		} else {
			set do_dataplot1 1
			set do_dataplot2 1
		}
	} else {
		set do_dataplot1 1
		set do_dataplot2 1
	}
	
	if {([file exists $eps3file]  && [file mtime $toifilename] < [file mtime $eps3file]) && [file mtime $datafilename] < [file mtime $eps3file]} {
		if {!([file exists $runlogfilename] && [file mtime $runlogfilename] > [file mtime $eps3file])} {
			set do_dataplot3 0
		} else {
			set do_dataplot3 1
		}
	} else {
		set do_dataplot3 1
	}

	if {$do_dataplot1 || $do_dataplot2 || $do_dataplot3} {	
		puts "  scanning data ($xbeg - $xend) $comment"	
		scan_data $basename $xbeg $xend "uhold amp2 frq2 wid1 wid2 lin1 lin2 bathtemp celltemp pressure baratron fil1res fil2res subsres coil_current coil_voltage pulsenumber"
	}
	
#
# Jetzt gehts ans Plotten
#
	if {$do_dataplot1} {
		puts "  plotting data ($xbeg - $xend) $comment"	
			
		set gnuplot [open "|gnuplot" w]
		incr opencount

		puts $gnuplot "set term post eps enhanced color solid"
		puts $gnuplot "set encoding iso"
		puts $gnuplot "set pointsize 0.5"
		puts $gnuplot "set xdata time"
		puts $gnuplot "set timefmt \"%d.%m.%y %H:%M:%S\""
		puts $gnuplot "set xr \[\"$xbeg\":\"$xend\"\]"
		puts $gnuplot "set output \"$eps1file\""

		if {$col(amp1)!=0 && $col(frq1)!=0 && $col(wid1)!=0} {
			puts $gnuplot "set multiplot"
			puts $gnuplot "set ytics nomirror"
			puts $gnuplot "set y2tics"
			puts $gnuplot "set rmargin 11"
			puts $gnuplot "set lmargin 10"
			puts $gnuplot "set size 1,0.5"
			puts $gnuplot "set origin 0,0.5"
			puts $gnuplot "set ylabel \"transmission \[dB\]\""
			puts $gnuplot "set auto y2"
			puts $gnuplot "set y2label \"frequency \[GHz\]\""
			if {$col(lin1)!=0 && $col(lin1:max)!=""} {
					puts $gnuplot "plot \"<bzcat $basename$dataext\" u 1:(\$$col(frq1)/1e9) axes x1y2 ti \"frequency\" w l $color(frq1),\
										\"\" u 1:(log(\$$col(amp1)+\$$col(off1)+\$$col(lin1)/1e9*\$$col(frq1)/1e9)/log(10)*20) ti \"transmission\" w l $color(tra1)"
			} else {
				puts $gnuplot "plot \"<bzcat $basename$dataext\" u 1:(\$$col(frq1)/1e9) axes x1y2 ti \"frequency\" w l $color(frq1),\
									\"\" u 1:(log(\$$col(amp1))/log(10)*20) ti \"transmission\" w l $color(tra1)"
			}
			puts $gnuplot "set origin 0,0"
			if {[file exists $runlogfilename]} {
				parse_runlog $gnuplot $basename $xbeg $xend $col(uhold:min) $uholdskip
			}
			if {$col(wid1:max)!=""} {
				puts $gnuplot "set ylabel \"quality factor\""
				puts $gnuplot "set y2label \"holding voltage \[V\]\""
				puts $gnuplot "set y2r \[$col(uhold:min):$col(uhold:max)\]"
				puts -nonewline $gnuplot "plot \"\" \
									u 1:(\$$col(frq1)/\$$col(wid1)) ti \"quality factor\" w l $color(q1)"
				if {$col(uhold:valid)} {
					puts -nonewline $gnuplot ", \"\" u 1:$col(uhold) axes x1y2 ti \"holding voltage\" w l $color(uhold)"
				}
				
			} else {
				puts $gnuplot "set y2label \"\""
				puts $gnuplot "set ylabel \"holding voltage \[V\]\""
				puts $gnuplot "set yr \[$col(uhold:min):$col(uhold:max)\]"
				puts $gnuplot "set ytics mirror"
				puts $gnuplot "set noy2tics"
				if {$col(uhold:valid)} {
					puts -nonewline $gnuplot "plot \"\" u 1:$col(uhold) axes x1y2 ti \"holding voltage\" w l $color(uhold)"
				}
			}
				
			if {$col(pulsenumber:max) > 0} {
				if {$col(uhold:valid)} {
					puts -nonewline $gnuplot ", "
				} else {
					if {$col(lin1:max)!=""} {
						puts -nonewline $gnuplot ", "
					} else {
						puts -nonewline $gnuplot "plot "
					}
				}
				if {$col(filused) != 0} {
					puts $gnuplot "\"\" u 1:($uholdskip*(-sgn(abs(\$$col(pulsenumber)))+1)+$col(uhold:max)-0.5*$uholdskip+(\$$col(filused)==1?0:$uholdskip)) axes x1y2 ti \"pulsing filament 1\" w p $color(pulse1) $pointstyle(pulse1), \\"
					puts $gnuplot "\"\" u 1:($uholdskip*(-sgn(abs(\$$col(pulsenumber)))+1)+$col(uhold:max)-0.5*$uholdskip+(\$$col(filused)==2?0:$uholdskip)) axes x1y2 ti \"pulsing filament 2\" w p $color(pulse2) $pointstyle(pulse2)"
				} else {
					puts $gnuplot "\"\" u 1:($uholdskip*(-sgn(abs(\$$col(pulsenumber)))+1)+$col(uhold:max)-0.5*$uholdskip) axes x1y2 ti \"pulsing the filament\" w p $color(pulse) $pointstyle(pulse)"
				}
			} else {
				puts $gnuplot ""
			}
			puts $gnuplot "set nomultiplot"
		} elseif {$col(dbmax1)!=0 && $col(frqmax1)!=0} {
				scan [file tail $basename] "mw%s" filenumber
				set basefitname [file join [file dirname $basename] "mw1_$filenumber"]
				
			if {$col(uhold)!=0} {
				puts $gnuplot "set multiplot"
				puts $gnuplot "set rmargin 11"
				puts $gnuplot "set lmargin 10"
				puts $gnuplot "set size 1,0.5"
				puts $gnuplot "set origin 0,0.5"
				puts $gnuplot "set ytics nomirror"
				puts $gnuplot "set y2tics"
				puts $gnuplot "set ylabel \"transmission \[dB\]\""
				puts $gnuplot "set y2label \"frequency \[GHz\]\""
				puts -nonewline $gnuplot "plot \"<bzcat $basename$dataext\" u 1:(\$$col(frqmax1)/1e9) axes x1y2 ti \"frequency\" w l $color(frq1),\
									\"\" u 1:$col(dbmax1) ti \"transmission\" w l $color(tra1)"
				if {[file exists "$basefitname.fitval"]} {
					puts $gnuplot ", \"$basefitname.fitval\" u 1:(\$6/1e9) axes x1y2 ti \"frequency (resonance fit)\" w l $color(frqf),\
								  \"\" u 1:(log(\$4)/log(10)*20) ti \"transmission (resonance fit)\" w l $color(traf)"
				} else {
					puts $gnuplot ""
				}
				puts $gnuplot "set origin 0,0"
				if {[file exists "$basefitname.fitval"]} {
					puts $gnuplot "set ytics nomirror"
					puts $gnuplot "set y2tics"
					puts $gnuplot "set ylabel \"quality factor\""
					puts $gnuplot "set y2label \"holding voltage\""
					puts $gnuplot "set y2r \[$col(uhold:min):$col(uhold:max)\]"
					if {[file exists $runlogfilename]} {
						parse_runlog $gnuplot $basename $xbeg $xend $col(uhold:min) $uholdskip
					}
					puts $gnuplot "plot \"$basefitname.fitval\" u 1:(\$6/\$8) ti \"quality factor (resonance fit)\" w l $color(q1),\
										\"<bzcat $basename$dataext\" u 1:$col(uhold) axes x1y2 ti \"holding voltage\" w l $color(uhold)"
					puts $gnuplot "set auto y2"
				} else {
					puts $gnuplot "set ylabel \"holding voltage \[V\]\""
					puts $gnuplot "set yr \[$col(uhold:min):$col(uhold:max)\]"
					if {[file exists $runlogfilename]} {
						parse_runlog $gnuplot $basename $xbeg $xend $col(uhold:min) $uholdskip 1
					}
					puts $gnuplot "set noy2tics"
					puts $gnuplot "set ytics mirror"
					puts $gnuplot "set y2label \"\""
					puts $gnuplot "plot \"<bzcat $basename$dataext\" u 1:$col(uhold) ti \"holding voltage\" w l $color(uhold)"
					puts $gnuplot "set auto y"
				}
				puts $gnuplot "set nomultiplot"
			} else {
				if {[file exists "$basefitname.fitval"]} {
					puts $gnuplot "set multiplot"
					puts $gnuplot "set rmargin 11"
					puts $gnuplot "set lmargin 10"
					puts $gnuplot "set size 1,0.5"
					puts $gnuplot "set origin 0,0"
					puts $gnuplot "set ylabel \"quality factor\""
					puts $gnuplot "plot \"$basefitname.fitval\" u 1:(abs(\$6/\$8)) ti \"quality factor (resonance fit)\" w l $color(qf)"
					puts $gnuplot "set auto y2"
					puts $gnuplot "set origin 0,0.5"
					puts $gnuplot "set ytics nomirror"
					puts $gnuplot "set y2tics"
					puts $gnuplot "set ylabel \"transmission \[dB\]\""
					puts $gnuplot "set y2label \"frequency \[GHz\]\""
					puts $gnuplot "plot \"<bzcat $basename$dataext\" u 1:$col(dbmax1) ti \"transmission\" w l $color(tra1),\
										\"\" u 1:(\$$col(frqmax1)/1e9) axes x1y2 ti \"frequency\" w l $color(frq1), \
										\"$basefitname.fitval\" u 1:(log(\$4)/log(10)*20) ti \"transmission (resonance fit)\" w l $color(traf),\
										\"\" u 1:(\$6/1e9) axes x1y2 ti \"frequency (resonance fit)\" w l $color(frqf)"
					puts $gnuplot "set nomultiplot"
				} else {
					puts $gnuplot "set rmargin 11"
					puts $gnuplot "set lmargin 10"
					puts $gnuplot "set ytics nomirror"
					puts $gnuplot "set y2tics"
					puts $gnuplot "set ylabel \"transmission \[dB\]\""
					puts $gnuplot "set y2label \"frequency \[GHz\]\""
					puts $gnuplot "plot \"<bzcat $basename$dataext\" u 1:(\$$col(frqmax1)/1e9) axes x1y2 ti \"frequency\" w l $color(frq1),\
										\"\" u 1:$col(dbmax1) ti \"transmission\" w l $color(tra1)"
				}
			}
			
		}
		catch {close $gnuplot} 
		incr opencount -1
	} else {
		puts "  skipping data plot ($xbeg - $xend) $comment"
	}
		
	if {$do_dataplot2} {
		if {$col(amp2)!=0 && $col(frq2)!=0 && $col(amp2:max)!=""} {
			puts "  plotting data2 ($xbeg - $xend) $comment"
			set gnuplot [open "|gnuplot" w]
			incr opencount

			puts $gnuplot "set term post eps enhanced color solid"
			puts $gnuplot "set encoding iso"
			puts $gnuplot "set pointsize 0.5"
			puts $gnuplot "set xdata time"
			puts $gnuplot "set timefmt \"%d.%m.%y %H:%M:%S\""
			puts $gnuplot "set xr \[\"$xbeg\":\"$xend\"\]"
			puts $gnuplot "set output \"$eps2file\""

			puts $gnuplot "set multiplot"
			puts $gnuplot "set ytics nomirror"
			puts $gnuplot "set y2tics"
			puts $gnuplot "set rmargin 11"
			puts $gnuplot "set lmargin 10"
			puts $gnuplot "set size 1,0.5"
			puts $gnuplot "set origin 0,0.5"
			puts $gnuplot "set ylabel \"transmission \[dB\]\""
			puts $gnuplot "set auto y2"
			puts $gnuplot "set y2label \"frequency \[GHz\]\""
			if {$col(lin2)!=0 && $col(lin2:max)!=""} {
					puts -nonewline $gnuplot "plot \"<bzcat $basename$dataext\" "
					if {$col(lin2:max)!=""} {
						puts -nonewline $gnuplot "u 1:(\$$col(frq2)/1e9) axes x1y2 ti \"frequency\" w l $color(frq2), \"\""
					}
					puts $gnuplot "u 1:(log(\$$col(amp2)+\$$col(off2)+\$$col(lin2)/1e9*\$$col(frq2)/1e9)/log(10)*20) ti \"transmission\" w l $color(tra2)"
			} else {
				puts $gnuplot "plot \"<bzcat $basename$dataext\" u 1:(\$$col(frq2)/1e9) axes x1y2 ti \"frequency\" w l $color(frq2),\
									\"\" u 1:(log(\$$col(amp2))/log(10)*20) ti \"transmission\" w l $color(tra2)"
			}
			puts $gnuplot "set origin 0,0"
			if {[file exists $runlogfilename]} {
				parse_runlog $gnuplot $basename $xbeg $xend $col(uhold:min) $uholdskip
			}
			if {$col(wid2:max)!=""} {
				puts $gnuplot "set ylabel \"quality factor\""
				puts $gnuplot "set y2label \"holding voltage \[V\]\""
				puts $gnuplot "set y2r \[$col(uhold:min):$col(uhold:max)\]"
				puts -nonewline $gnuplot "plot \"\" \
									u 1:(\$$col(frq2)/\$$col(wid2)) ti \"quality factor\" w l $color(q2)"
				if {$col(uhold:valid)} {
					puts -nonewline $gnuplot ", \"\" u 1:$col(uhold) axes x1y2 ti \"holding voltage\" w l $color(uhold)"
				}
			} else {
				puts $gnuplot "set y2label \"\""
				puts $gnuplot "set ylabel \"holding voltage \[V\]\""
				puts $gnuplot "set yr \[$col(uhold:min):$col(uhold:max)\]"
				puts $gnuplot "set ytics mirror"
				puts $gnuplot "set noy2tics"
				if {$col(uhold:valid)} {
					puts -nonewline $gnuplot "plot \"\" u 1:$col(uhold) axes x1y2 ti \"holding voltage\" w l $color(uhold)"
				}
			}
				
			if {$col(pulsenumber) != 0} {
				if {$col(uhold:valid)} {
					puts -nonewline $gnuplot ", "
				} else {
					if {$col(lin2:max)!=""} {
						puts -nonewline $gnuplot ", "
					} else {
						puts -nonewline $gnuplot "plot "
					}
				}
				if {$col(filused) != 0} {
					puts $gnuplot "\"\" u 1:($uholdskip*(-sgn(abs(\$$col(pulsenumber)))+1)+$col(uhold:max)-0.5*$uholdskip+(\$$col(filused)==1?0:$uholdskip)) axes x1y2 ti \"pulsing filament 1\" w p $color(pulse1) $pointstyle(pulse1), \\"
					puts $gnuplot "\"\" u 1:($uholdskip*(-sgn(abs(\$$col(pulsenumber)))+1)+$col(uhold:max)-0.5*$uholdskip+(\$$col(filused)==2?0:$uholdskip)) axes x1y2 ti \"pulsing filament 2\" w p $color(pulse2) $pointstyle(pulse2)"
				} else {
					puts $gnuplot "\"\" u 1:($uholdskip*(-sgn(abs(\$$col(pulsenumber)))+1)+$col(uhold:max)-0.5*$uholdskip) axes x1y2 ti \"pulsing the filament\" w p $color(pulse) $pointstyle(pulse)"
				}
			} else {
				puts $gnuplot ""
			}
			puts $gnuplot "set nomultiplot"
			catch {close $gnuplot} 
			incr opencount -1
		} else {
			puts "  no data2 plot ($xbeg - $xend) $comment"
		}
	} else {
		puts "  skipping data2 plot ($xbeg - $xend) $comment"
	}
	
	if {$do_dataplot3} {
		puts "  plotting data3 ($xbeg - $xend) $comment"	
			
		set gnuplot [open "|gnuplot" w]
		incr opencount

		puts $gnuplot "set term post eps enhanced color solid"
		puts $gnuplot "set encoding iso"
		puts $gnuplot "set pointsize 0.5"
		puts $gnuplot "set xdata time"
		puts $gnuplot "set timefmt \"%d.%m.%y %H:%M:%S\""
		puts $gnuplot "set xr \[\"$xbeg\":\"$xend\"\]"
		puts $gnuplot "set output \"$eps3file\""
		puts $gnuplot "set rmargin 11"
		puts $gnuplot "set lmargin 10"
		if {$col(fil1res:max)!=""} {
			puts $gnuplot "set multiplot"
			puts $gnuplot "set origin 0,0.5"
			puts $gnuplot "set size 1,0.5"
			puts $gnuplot "set ylabel \"filament resistance \[{/Symbol W}\]\""
			if {$col(subsres:max)!=""} {
				puts $gnuplot "set y2tics"
				puts $gnuplot "set ytics nomirror"
				puts $gnuplot "set log y2"
				puts $gnuplot "set y2label \"substrate resistance \[k{/Symbol W}\]\""
				puts -nonewline $gnuplot "plot \"<bzcat $basename$dataext\" u 1:(\$$col(subsres)/1e3) axes x1y2 ti \"substrate resistance\" w l $color(subsres), "
			} else {
				puts -nonewline $gnuplot "plot "
			}
			puts $gnuplot "\"<bzcat $basename$dataext\" u 1:$col(fil1res) ti \"filament 1\" w l, \"\" u 1:$col(fil2res) ti \"filament 2\" w l $color(fil2res)" 
			puts $gnuplot "set origin 0,0"
		} elseif {$col(coil_current:max)!=""} {
			puts $gnuplot "set multiplot"
			puts $gnuplot "set origin 0,0.5"
			puts $gnuplot "set size 1,0.5"
			puts $gnuplot "set ylabel \"coil current \[A\]\""
			if {$col(coil_voltage:max)!=""} {
				puts $gnuplot "set y2tics"
				puts $gnuplot "set ytics nomirror"
				puts $gnuplot "set y2label \"coil voltage \[V\]\""
				puts -nonewline $gnuplot "plot \"<bzcat $basename$dataext\" u 1:$col(coil_voltage) axes x1y2 ti \"coil voltage\" w l $color(coilvolt), "
			} else {
				puts -nonewline $gnuplot "plot "
			}
			puts $gnuplot "\"<bzcat $basename$dataext\" u 1:$col(coil_current) ti \"coil current\" w l $color(coilamp)"
			puts $gnuplot "set origin 0,0"
		}

		puts $gnuplot "set ylabel \"temperature sensors \[k{/Symbol W}\]\""
		if {$col(pressure:max)!=""} {
				puts $gnuplot "set y2tics"
				puts $gnuplot "set ytics nomirror"
				puts $gnuplot "set nolog y2"
				puts $gnuplot "set y2label \"cell pressure \[hPa\]\""
				if {$col(pressure:max)>0.3} {
					puts -nonewline $gnuplot "plot \"<bzcat $basename$dataext\" u 1:(35.2254582566528*\$$col(pressure)-0.110233669877985) axes x1y2 ti \"panel pressure\" w l $color(pressure), "
				} else {
					if {$col(baratron:max)!=""} {
						puts -nonewline $gnuplot "plot "
					} else {
						puts -nonewline $gnuplot "plot \"<bzcat $basename$dataext\" u 1:(35.2254582566528*\$$col(pressure)-0.110233669877985) axes x1y2 ti \"panel pressure\" w l $color(pressure), "
					}
				}
				if {$col(baratron:max)!=""} {
					puts -nonewline $gnuplot "\"<bzcat $basename$dataext\" u 1:(1.33444915413664*abs(\$$col(baratron))+0.150784483171523626) axes x1y2 ti \"baratron pressure\" w l $color(baratron), "
				}
		} else {
				puts $gnuplot "set noy2tics"
				puts $gnuplot "set ytics mirror"
				puts $gnuplot "set nolog y2"
				puts $gnuplot "set y2label \"\""
				puts -nonewline $gnuplot "plot "
		}
		
		puts $gnuplot "\"<bzcat $basename$dataext\" u 1:(\$$col(bathtemp)/1e3) ti \"bath temperature\" w l $color(bathtemp), \"\" u 1:(\$$col(celltemp)/1e3) ti \"cell temperature\" w l $color(celltemp)"

		if {$col(fil1res:max)!=""} {
			puts $gnuplot "set nomultiplot"
		}
		catch {close $gnuplot}
		incr opencount -1
	} else {
		puts "  skipping data3 plot ($xbeg - $xend) $comment"
	}
	return
}

foreach toifilename $argv {
	
	if {![file exists $toifilename]} {
		puts "ERROR: file '$toifilename' does not exist"
	}

	puts "working on '$toifilename' ..."
	
	set basename [file rootname $toifilename]

	if {[file exists $basename$dataext]} {
		set toifile [open $toifilename "r"]
		incr opencount
		
		gets $toifile toiline
		set xbeg [lindex $toiline 0]
		set xend [lindex $toiline 1]
		set comment [lindex $toiline 2]

		set plotnumber 0
		
		plot_level $basename $xbeg $xend
		plot_levelcalib $basename $toiline
		plot_transres $basename
		plot_data $basename $xbeg $xend $comment

		while {![eof $toifile]} {			
			gets $toifile toiline
			set toiline [string trim $toiline]

			if {$toiline!="" && [string first # $toiline]!=0} {
				set xbeg [lindex $toiline 0]
				set xend [lindex $toiline 1]
				set comment [lindex $toiline 2]
				incr plotnumber
				set plotindex 0
				plot_data $basename $xbeg $xend $comment
			
				while {[llength $toiline] >= ($plotindex+2)*3} {
					incr plotindex
					if {[lindex $toiline [expr 3*$plotindex]]!=0 || [lindex $toiline [expr 3*$plotindex+1]]!=0} { 
						set xbeg [lindex $toiline [expr 3*$plotindex]]
						set xend [lindex $toiline [expr 3*$plotindex+1]]
					}
					set comment [lindex $toiline [expr 3*$plotindex+2]]
					if {[string first "CR2" $comment]==0} {
						plot_CR2 $basename $xbeg $xend $comment
					} elseif {[string first "CR" $comment]==0} {
						plot_CR $basename $xbeg $xend $comment
					} elseif {[string first "AB2" $comment]==0} {
						plot_AB2 $basename $xbeg $xend $comment
					} elseif {[string first "AB" $comment]==0} {
						plot_AB $basename $xbeg $xend $comment
					} elseif {[string first "LC" $comment]==0} {
						plot_LC $basename $xbeg $xend $comment
					} elseif {[string first "CD" $comment]==0} {
						plot_CD $basename $xbeg $xend $comment
					} elseif {[string first "PO" $comment]==0} {
						plot_PO $basename $xbeg $xend $comment
					} 
				}
			}
		}
		close $toifile
		incr opencount -1
	}
}
