#!/usr/bin/tclsh

source ~andi/bin/fileformat.tcl

set wwwpicdir "pics"
set iconpicdpi 50
set picdpi 200
set dataext ".dat.bz2"
set sectiontitle "H2"
set contentslink "H3"
set taboptions ""
set celloptions " valign=\"top\""
set htmlhead "<meta http-equiv=\"content-type\" content=\"text/html; charset=iso-8859-1\">\n\
<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\" \"http://www.w3.org/MarkUp/Wilbur/HTML32.dtd\">\n\
<HTML>\n\
<head>\n\
<meta http-equiv=\"content-type\" content=\"text/html; charset=iso-8859-1\">\n"

proc get_columns {{basename ""}} {
	global dataext
	set datafile [open "|bzcat $basename$dataext" "r"];
	gets $datafile dataline
	set columns [llength $dataline]
	catch {close $datafile}
	return $columns
}

proc stamptotime {{timestamp ""}} {
	if {[ catch { scan $timestamp "%d.%d.%d %d:%d:%d" day month year hours minutes seconds}]!=0} {
		puts "$timestamp"
	}
	if {[catch { set result [clock scan [format "%02d/%02d/%02d %02d:%02d:%02d" $month $day $year $hours $minutes $seconds]]}] != 0} {
		puts "ERROR: $timestamp"
		exit
	}
	return $result
}

proc parse_runlog {{basename ""} {pichtmlfile ""} {plotnumber ""} {xbeg ""} {xend ""}} {
	if {[file exists "$basename.runlog"]} {
		set starttime [stamptotime $xbeg]
		set stoptime [stamptotime $xend]
		set entrycount 0
		
		set runlogfile [open "$basename.runlog" r]
		set loglines ""
		
		while {![eof $runlogfile]} {
			gets $runlogfile runlogline
				
			if {[string trim $runlogline]!=""} { 
				set messagetime "[lindex $runlogline 2] [lindex $runlogline 3]"
				set messagesecs [stamptotime $messagetime]

				if {$messagesecs > $starttime && $messagesecs < $stoptime} {
					incr entrycount
					set loglines "$loglines$messagetime \"[lindex $runlogline 4]\"<br>\n"
				}
			}
		}
		if {$entrycount>0} {
			puts $pichtmlfile "<H4>Logfile</H4>"
			puts $pichtmlfile $loglines
		}
		close $runlogfile
	}		
}

proc plot_level {{basename ""} {xbeg ""} {xend ""} {htmlfile ""}} {
	global wwwpicdir iconpicdpi picdpi sectiontitle dataext taboptions celloptions
# Diese Subroutine ist fr das Plotten des Heliumlevels zustndig
	set columns [get_columns $basename]
	
	set epsfile "$basename[]_level.eps"
	set eps2file "$basename[]_levelcalib.eps"
	
	if {[get_column $columns "levelcap"]==0} {
		if {![file exists $eps2file]} {
			return
		}
	}

	puts $htmlfile "\n<$sectiontitle>helium level ($xbeg - $xend)</$sectiontitle>"
	puts $htmlfile "<p><table$taboptions><tr>"

	if {[file exists $epsfile]} {	
		set pngiconfile [file join [file dirname $basename] $wwwpicdir [file tail $basename]_level_icon.png]
		set pngfile [file join [file dirname $basename] $wwwpicdir [file tail $basename]_level.png]
		set pdffile [file join [file dirname $basename] [file tail $basename]_level.pdf]

		if {![file exists [file join [file dirname $basename] $wwwpicdir]]} {
			file mkdir [file join [file dirname $basename] $wwwpicdir]
			file attributes [file join [file dirname $basename] $wwwpicdir] -permissions 0755
		}

		if {![file exists $pngfile] || [file mtime $pngfile] < [file mtime $epsfile]} {
			puts "  converting level ($xbeg - $xend) -> png"
			catch {exec convert -density $picdpi[]x$picdpi $epsfile png:$pngfile}
			catch {exec epstopdf $epsfile}
			catch {file attributes $pngfile -permissions 0644}
			catch {file attributes $epsfile -permissions 0644}
			catch {file attributes $pdffile -permissions 0644}
		}
		
		if {![file exists $pngiconfile] || [file mtime $pngiconfile] < [file mtime $epsfile]} {
			puts "  converting level ($xbeg - $xend) -> png_icon"
			catch {exec convert -density $iconpicdpi[]x$iconpicdpi $epsfile png:$pngiconfile}
			catch {file attributes $pngiconfile -permissions 0644}
		}

		if {[file exists $pngfile]} {
			puts $htmlfile "  <td$celloptions><A HREF=\"[file join $wwwpicdir [file tail $pngfile]]\"><IMG SRC=\"[file join $wwwpicdir [file tail $pngiconfile]]\"></A><br>"
			puts $htmlfile "    <A HREF=\"[file tail $epsfile]\">\[eps\]</A><A HREF=\"[file tail $pdffile]\">\[pdf\]</A></td>"
		}
	}

	
	if {[file exists $eps2file]} {
		set png2iconfile [file join [file dirname $basename] $wwwpicdir [file tail $basename]_levelcalib_icon.png]
		set png2file [file join [file dirname $basename] $wwwpicdir [file tail $basename]_levelcalib.png]
		set pdf2file [file join [file dirname $basename] [file tail $basename]_levelcalib.pdf]
		
		if {![file exists $png2file] || [file mtime $png2file] < [file mtime $eps2file]} {
			puts "  converting levelcalib ($xbeg - $xend) -> png"
			catch {exec convert -density $picdpi[]x$picdpi $eps2file png:$png2file}
			catch {exec epstopdf $eps2file}
			catch {file attributes $png2file -permissions 0644}
			catch {file attributes $eps2file -permissions 0644}
			catch {file attributes $pdf2file -permissions 0644}
		}
		
		if {![file exists $png2iconfile] || [file mtime $png2iconfile] < [file mtime $eps2file]} {
			puts "  converting levelcalib ($xbeg - $xend) -> png_icon"
			catch {exec convert -density $iconpicdpi[]x$iconpicdpi $eps2file png:$png2iconfile}
			catch {file attributes $png2iconfile -permissions 0644}
		}
		if {[file exists $png2file]} {
			puts $htmlfile "  <td$celloptions><A HREF=\"[file join $wwwpicdir [file tail $png2file]]\"><IMG SRC=\"[file join $wwwpicdir [file tail $png2iconfile]]\"></A><br>"
			puts $htmlfile "<A HREF=\"[file tail $eps2file]\">\[eps\]</A><A HREF=\"[file tail $pdf2file]\">\[pdf\]</A></td>"
		}
	}
#	puts $htmlfile "  <td ALIGN=left$celloptions></td>"
	puts $htmlfile "</tr></table></p>"
}

proc plot_data {{basename ""} {xbeg ""} {xend ""} {htmlfile ""} {plotnumber ""} {comment ""}} {
	global wwwpicdir iconpicdpi picdpi substratenr sectiontitle dataext taboptions celloptions
	if {$plotnumber==0} {
		foreach i "a b c d" {
			set epsfile($i) "$basename[]_$i.eps"
			set pngiconfile($i) [file join [file dirname $basename] $wwwpicdir [file tail $basename]_$i[]_icon.png]
			set pngfile($i) [file join [file dirname $basename] $wwwpicdir [file tail $basename]_$i[].png]
			set pdffile($i) [file join [file dirname $basename] [file tail $basename]_$i[].pdf]
		}
	} else {
		foreach i "a b c d" {
			set epsfile($i) "$basename[]_[format "%03d" $plotnumber]_$i.eps"
			set pngiconfile($i) [file join [file dirname $basename] $wwwpicdir [file tail $basename]_[format "%03d" $plotnumber]_$i[]_icon.png]
			set pngfile($i) [file join [file dirname $basename] $wwwpicdir [file tail $basename]_[format "%03d" $plotnumber]_$i[].png]
			set pdffile($i) [file join [file dirname $basename] [file tail $basename]_[format "%03d" $plotnumber]_$i[].pdf]
		}
	}
	set pichtmlfilename "$basename[]_[format "%03d" $plotnumber].html"
	
	if {![file exists [file join [file dirname $basename] $wwwpicdir]]} {
		file mkdir [file join [file dirname $basename] $wwwpicdir]
		file attributes [file join [file dirname $basename] $wwwpicdir] -permissions 0755
	}
	
	foreach i "a b c d" {
		if {[file exists $epsfile($i)]} {
			if {![file exists $pngfile($i)] || [file mtime $pngfile($i)] < [file mtime $epsfile($i)]} {
				puts "  converting data $i ($xbeg - $xend) -> png"
				catch {exec convert -density $picdpi[]x$picdpi $epsfile($i) png:$pngfile($i)}
				catch {exec epstopdf $epsfile($i)}
				catch {file attributes $pngfile($i) -permissions 0644}
				catch {file attributes $epsfile($i) -permissions 0644}
				catch {file attributes $pdffile($i) -permissions 0644}
			}
			
			if {![file exists $pngiconfile($i)] || [file mtime $pngiconfile($i)] < [file mtime $epsfile($i)]} {
				puts "  converting data $i ($xbeg - $xend) -> png_icon"
				catch {exec convert -density $iconpicdpi[]x$iconpicdpi $epsfile($i) png:$pngiconfile($i)}
				catch {file attributes $pngiconfile($i) -permissions 0644}
			}
		}
	}

	if {$plotnumber == 0} {	
		puts $htmlfile "<$sectiontitle>measurement overview ($xbeg - $xend)</$sectiontitle>"
		puts $htmlfile "<A HREF=\"[file tail $basename]$dataext\">datafile</A> has [get_columns $basename] columns<br>"

		set runlogfile "$basename.runlog"
		if {[file exists $runlogfile]} {
			file attributes $runlogfile -permissions 0644
			puts $htmlfile "<A HREF=\"[file tail $runlogfile]\">logfile</A><br>"
		}
		
		set infofile "$basename.info"
		if {[file exists $infofile]} {
			file attributes $infofile -permissions 0644
			puts $htmlfile "<A HREF=\"[file tail $infofile]\">measurement information</A><br>"
			set substratenr ""	
			set infofilefp [open $infofile r]
			while {![eof $infofilefp]} {
				gets $infofilefp infoline
				if {[string trim $infoline]!=""} {
					if {[lindex $infoline 0]=="substrate"} {
						set substratenr [lindex $infoline 1]
						set substratepath ".."
						for {set i 1} {$i <= [llength [file split [file dirname $basename]]]} {incr i} {
							set substratepath [file join $substratepath ".."]
						}
						exec ln -fsn "[file join $substratepath substrates $substratenr]" "[file dirname $basename]/substrate"
						puts $htmlfile "<A HREF=\"substrate/index.html\">substrate #$substratenr information</A><br>"
					}
				}
			}
			close $infofilefp
		} else {
			set substratenr ""
		}

		puts $htmlfile "<p><table$taboptions><tr>"
		puts $htmlfile "  <td$celloptions><A HREF=\"[file tail $pichtmlfilename]\"><IMG SRC=\"[file join $wwwpicdir [file tail $pngiconfile(a)]]\"></A></td>"
		
		foreach i "b c d" {		
			if {[file exists $pngiconfile($i)]} {
				puts $htmlfile "  <td$celloptions><A HREF=\"[file join $wwwpicdir [file tail $pngfile($i)]]\"><IMG SRC=\"[file join $wwwpicdir [file tail $pngiconfile($i)]]\"></A><br>"
				puts $htmlfile "    <A HREF=\"[file tail $epsfile($i)]\">\[eps\]</A><A HREF=\"[file tail $pdffile($i)]\">\[pdf\]</A></td>"
			}
		}
		puts $htmlfile "</tr></table></p>"
	
		set pichtmlfile [open $pichtmlfilename w]
		puts $pichtmlfile "<HTML><TITLE>[file tail $basename] #$plotnumber</TITLE><BODY>"	
		puts $pichtmlfile "<$sectiontitle>[file tail $basename] #$plotnumber ($xbeg - $xend)"
		if {$comment != ""} {
			puts $pichtmlfile " \"$comment\"</$sectiontitle>"
		} else {
			puts $pichtmlfile "</$sectiontitle>"
		}	
		puts $pichtmlfile "<IMG SRC=\"[file join $wwwpicdir [file tail $pngfile(a)]]\"><br><A HREF=\"[file tail $epsfile(a)]\">\[eps\]</A><A HREF=\"[file tail $pdffile(a)]\">\[pdf\]</A><br></td>"
		parse_runlog $basename $pichtmlfile $plotnumber $xbeg $xend
   		close $pichtmlfile

		file attributes $pichtmlfilename -permissions 0644
	} else {
		puts $htmlfile	"\n<$sectiontitle>part #$plotnumber ($xbeg - $xend)</$sectiontitle>"
		puts $htmlfile "<p><table$taboptions><tr>"
		puts -nonewline $htmlfile "  <td$celloptions><A HREF=\"[file tail $pichtmlfilename]\">\
						<IMG SRC=\"[file join $wwwpicdir [file tail $pngiconfile(a)]]\"></A>"
		if {$comment != ""} {
			puts $htmlfile "\n    <br>$comment</td>"
		} else {
			puts $htmlfile "</td>"
		}

		foreach i "b c" {		
			if {[file exists $pngiconfile($i)]} {
				puts $htmlfile "  <td$celloptions><A HREF=\"[file join $wwwpicdir [file tail $pngfile($i)]]\"><IMG SRC=\"[file join $wwwpicdir [file tail $pngiconfile($i)]]\"></A><br>"
				puts $htmlfile "    <A HREF=\"[file tail $epsfile($i)]\">\[eps\]</A><A HREF=\"[file tail $pdffile($i)]\">\[pdf\]</A></td>"
			}
		}
		
		set pichtmlfile [open $pichtmlfilename w]
		puts $pichtmlfile "<HTML><TITLE>[file tail $basename] #$plotnumber </TITLE><BODY>"	
		puts $pichtmlfile "<$sectiontitle>[file tail $basename] #$plotnumber ($xbeg - $xend)"	
		if {$comment != ""} {
			puts $pichtmlfile " \"$comment\"</$sectiontitle>"
		} else {
			puts $pichtmlfile "</$sectiontitle>"
		}	
		puts $pichtmlfile "<IMG SRC=\"[file join $wwwpicdir [file tail $pngfile(a)]]\"><br>"
		puts $pichtmlfile "<A HREF=\"[file tail $epsfile(a)]\">\[eps\]</A><A HREF=\"[file tail $pdffile(a)]\">\[pdf\]</A><br>"
		parse_runlog $basename $pichtmlfile $plotnumber $xbeg $xend
   		close $pichtmlfile
		file attributes $pichtmlfilename -permissions 0644
	}
}

proc plot_subdata {{basename ""} {xbeg ""} {xend ""} {htmlfile ""} {plotnumber ""} {picindex ""} {comment ""}} {
	global wwwpicdir iconpicdpi picdpi taboptions celloptions

	set subepsfile "$basename[]_[format "%03d%02d" $plotnumber $picindex].eps"
	set subpdffile "$basename[]_[format "%03d%02d" $plotnumber $picindex].pdf"
	set subpngfile [file join [file dirname $basename] $wwwpicdir [file tail $basename]_[format "%03d%02d" $plotnumber $picindex].png]
	set subpngiconfile [file join [file dirname $basename] $wwwpicdir [file tail $basename]_[format "%03d%02d" $plotnumber $picindex]_icon.png]
		
	if {![file exists $subpngfile] || [file mtime $subpngfile] < [file mtime $subepsfile]} {
		puts "    converting subdata #$picindex ($xbeg - $xend) -> png"
		catch {exec convert -density $picdpi[]x$picdpi $subepsfile png:$subpngfile}
		catch {exec epstopdf $subepsfile}
		catch {file attributes $subpngfile -permissions 0644}
		catch {file attributes $subepsfile -permissions 0644}
		catch {file attributes $subpdffile -permissions 0644}
	}
		
	if {![file exists $subpngiconfile] || [file mtime $subpngiconfile] < [file mtime $subepsfile]} {
		puts "    converting subdata #$picindex ($xbeg - $xend) -> png_icon"
		catch {exec convert -density $iconpicdpi[]x$iconpicdpi $subepsfile png:$subpngiconfile}
		catch {file attributes $subpngiconfile -permissions 0644}
	}

	puts $htmlfile	"  <td$celloptions><A HREF=\"[file join $wwwpicdir [file tail $subpngfile]]\"> <IMG SRC=\"[file join $wwwpicdir [file tail $subpngiconfile]]\"></A><br>"
	puts $htmlfile "$comment ($xbeg - $xend) <A HREF=\"[file tail $subepsfile]\">\[eps\]</A><A HREF=\"[file tail $subpdffile]\">\[pdf\]</A></td>"
}

proc plot_enddata {{htmlfile ""}} {
	puts $htmlfile "</tr></table></p>"
}

set master_htmlfile [open "index.html" w]
puts $master_htmlfile $htmlhead
puts $master_htmlfile "<TITLE>EMINUS measurement catalog</TITLE>"
puts $master_htmlfile "</head><BODY>"

set toifilelist [lsort -decreasing $argv]
foreach toifile $toifilelist {
	set commentlist ""

	if {![file exists $toifile]} {
		puts "ERROR: file '$toifile' does not exist"
	}

	set basename [file rootname $toifile]	
	if {[file exists $basename$dataext]} {
		puts "working on '$toifile' ..."
		
		set toifp [open $toifile r]
		set htmlfile [open "$basename.html" w]
		puts $htmlfile "$htmlhead<TITLE>[file tail $basename]</TITLE></head>"
		puts $htmlfile "<BODY>"
		
		gets $toifp toiline
		set xbeg [lindex $toiline 0]
		set masterxbeg $xbeg
		set xend [lindex $toiline 1]
		set masterxend $xend
		set comment [lindex $toiline 2]
		
		set plotnumber 0
		plot_data $basename $xbeg $xend $htmlfile $plotnumber $comment
		plot_level $basename $xbeg $xend $htmlfile

		while {![eof $toifp]} {
			gets $toifp toiline
			set toiline [string trim $toiline]
			if {$toiline!="" && [string first # $toiline]!=0} {
				set xbeg [lindex $toiline 0]
				set xend [lindex $toiline 1]
				set comment [lindex $toiline 2]

				if {$comment != ""} {
					if {$commentlist != ""} {
						set commentlist "$commentlist, $comment"
					} else {
						set commentlist $comment
					}
				}
				
				incr plotnumber
				set plotindex 0
				plot_data $basename $xbeg $xend $htmlfile $plotnumber $comment

				while {[llength $toiline] >= ($plotindex+2)*3} {
					incr plotindex
					set xbeg [lindex $toiline [expr 3*$plotindex]]
					set xend [lindex $toiline [expr 3*$plotindex+1]]
					set comment [lindex $toiline [expr 3*$plotindex+2]]
					plot_subdata $basename $xbeg $xend $htmlfile $plotnumber $plotindex $comment
				}
				plot_enddata $htmlfile 
			}
		}

		puts $htmlfile "</BODY></HTML>"
		close $htmlfile
		file attributes "$basename.html" -permissions 0644
		file attributes "$basename$dataext" -permissions 0644
		file attributes "[file dirname $basename]" -permissions 0755
		close $toifp
		
		puts $master_htmlfile "<A HREF=\"$basename.html\">\
						<$contentslink>[file tail $basename]</$contentslink></A>($masterxbeg - $masterxend)"
		if {$substratenr!=""} {
			 puts $master_htmlfile " substrate #$substratenr"
		}
		if {$commentlist!=""} {
			puts $master_htmlfile "<br>$commentlist<br>"
		} else {
			puts $master_htmlfile "<br>"
		}
	}
	flush $master_htmlfile
}

puts $master_htmlfile "</BODY></HTML>"
close $master_htmlfile

file attributes "index.html" -permissions 0644
